@@ -54,7 +54,7 @@ nlmixrBounds <- function(fun){
         if (is.name(x)) {
             character()
         } else if (is.call(x)) {
-            nenv <- new.env();
+            nenv <- new.env(parent=emptyenv());
             nenv$do.fixed <- FALSE;
             fix <- FIX <- fixed <- FIXED <- function(x){
                 assign("do.fixed", TRUE, nenv)
@@ -759,7 +759,7 @@ nlmixrBoundsOmega <- function(x, nlme=FALSE){
                     mat[neta1, neta2] <- mat[neta2, neta1]<- df$est[i];
                 }
             }
-            if (nlme){
+            if (is(nlme, "logical") && nlme){
                 df.diag <- df[df$neta1 == df$neta2, ];
                 n2 <- sprintf(".eta.%d", seq_along(df.diag$name));
                 w <- which(!is.na(df.diag$name))
@@ -770,6 +770,20 @@ nlmixrBoundsOmega <- function(x, nlme=FALSE){
                 } else {
                     return(nlme::pdSymm(as.matrix(Matrix::nearPD(mat)$mat), form=frm))
                 }
+            } else if (is(nlme, "list")){
+                df.diag <- df[df$neta1 == df$neta2, ];
+                class(df.diag) <- "data.frame"
+                n2 <- sprintf(".eta.%d", seq_along(df.diag$name));
+                w <- which(!is.na(df.diag$name))
+                n2[w] <- sapply(as.character(df.diag$name[w]), function(x){
+                    nlme[[x]]
+                })
+                frm <- as.formula(paste(paste(n2, collapse=" + "), "~ 1"))
+                if (diag){
+                    return(nlme::pdDiag(mat, form=frm))
+                } else {
+                    return(nlme::pdSymm(as.matrix(Matrix::nearPD(mat)$mat), form=frm))
+                }
             }
             return(mat);
         } else {
